/*
 * World Storage API
 *
 * API ensuring interoperability between an authoring tool and a World Storage service
 *
 * The version of the OpenAPI document: 1.0.0
 * 
 * Generated by: https://openapi-generator.tech
 */

using System;
using System.IO;
using System.Reflection;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.OpenApi.Models;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;
using ETSI.ARF.OpenAPI.WorldStorage.Authentication;
using ETSI.ARF.OpenAPI.WorldStorage.Filters;
using ETSI.ARF.OpenAPI.WorldStorage.OpenApi;
using ETSI.ARF.OpenAPI.WorldStorage.Formatters;
using ETSI.ARF.OpenAPI.WorldStorage.Services;
using MongoDB.Bson.Serialization.Serializers;
using MongoDB.Bson.Serialization;
using MongoDB.Bson;
using Microsoft.Extensions.Options;

namespace ETSI.ARF.OpenAPI.WorldStorage
{
    /// <summary>
    /// Startup
    /// </summary>
    public class Startup
    {
        //
        // SylR, Fraunhofer HHI
        //
        /// <summary>
        /// The API version. (how to read it from the yaml?)
        /// </summary>
        static public string apiVersion = "1.1.0";

        /// <summary>
        /// Demo access key
        /// </summary>
        static public string accessKey = "My!Key.ETSI";

        /// <summary>
        /// Demo secret key
        /// </summary>
        static public string secretKey = "GW0Wae1t4Cs5rAqEbPYFWO9J5nSbpJXxp1F3uv0J";

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="configuration"></param>
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        /// <summary>
        /// The application configuration.
        /// </summary>
        public IConfiguration Configuration { get; }

        /// <summary>
        /// SylR: Check if the request is authorized
        /// </summary>
        /// <param name="security"></param>
        /// <returns></returns>
        //static public bool IsAccessAllowed(ETSI.ARF.OpenAPI.WorldStorage.Models.SecureAccess security)
        //{
        //    Console.WriteLine(security.AccessKey);
        //    return true;
        //    //return (security.AccessKey == accessKey && security.SecretKey == secretKey);
        //}

        /// <summary>
        /// This method gets called by the runtime. Use this method to add services to the container.
        /// </summary>
        /// <param name="services"></param>
        public void ConfigureServices(IServiceCollection services)
        {
            //
            // SylR, Fraunhofer HHI
            //

            // appsetting.json - requires using Microsoft.Extensions.Options
            IServiceCollection c = services.Configure<DatabaseSettings>(Configuration.GetSection(nameof(DatabaseSettings)));
            
            //
            // MongoDB
            //
            BsonSerializer.RegisterSerializer(new GuidSerializer(GuidRepresentation.Standard));
            //BsonDefaults.GuidRepresentationMode = GuidRepresentationMode.V3;

            var s = services.AddSingleton<IDatabaseSettings>(sp => sp.GetRequiredService<IOptions<DatabaseSettings>>().Value);
            services.AddSingleton<TrackableService>();
            services.AddSingleton<WorldAnchorService>();
            services.AddSingleton<WorldLinkService>();
            
            // Add framework services.
            services
                // Don't need the full MVC stack for an API, see https://andrewlock.net/comparing-startup-between-the-asp-net-core-3-templates/
                .AddControllers(options => {
                    options.InputFormatters.Insert(0, new InputFormatterStream());
                })
                .AddNewtonsoftJson(opts =>
                {
                    opts.SerializerSettings.ContractResolver = new CamelCasePropertyNamesContractResolver();
                    opts.SerializerSettings.Converters.Add(new StringEnumConverter
                    {
                        NamingStrategy = new CamelCaseNamingStrategy()
                    });
                });
            services
                .AddSwaggerGen(c =>
                {
                    c.EnableAnnotations(enableAnnotationsForInheritance: true, enableAnnotationsForPolymorphism: true);
                    
                    c.SwaggerDoc(apiVersion, new OpenApiInfo
                    {
                        Title = "World Storage API",
                        Description = "World Storage API (ASP.NET Core 5.0)",
                        TermsOfService = new Uri("https://github.com/openapitools/openapi-generator"),
                        Contact = new OpenApiContact
                        {
                            Name = "OpenAPI-Generator Contributors",
                            Url = new Uri("https://github.com/openapitools/openapi-generator"),
                            Email = ""
                        },
                        License = new OpenApiLicense
                        {
                            Name = "NoLicense",
                            Url = new Uri("https://opensource.org/licenses/BSD-3-Clause")
                        },
                        Version = apiVersion,
                    });
                    c.CustomSchemaIds(type => type.FriendlyId(true));
                    c.IncludeXmlComments($"{AppContext.BaseDirectory}{Path.DirectorySeparatorChar}{Assembly.GetEntryAssembly().GetName().Name}.xml");

                    // Include DataAnnotation attributes on Controller Action parameters as OpenAPI validation rules (e.g required, pattern, ..)
                    // Use [ValidateModelState] on Actions to actually validate it in C# as well!
                    c.OperationFilter<GeneratePathParamsValidationFilter>();
                });
                services
                    .AddSwaggerGenNewtonsoftSupport();
        }

        /// <summary>
        /// This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        /// </summary>
        /// <param name="app"></param>
        /// <param name="env"></param>
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            //
            // Initialize all ARF services
            //
            app.ApplicationServices.GetRequiredService<TrackableService>();
            app.ApplicationServices.GetRequiredService<WorldAnchorService>();
            app.ApplicationServices.GetRequiredService<WorldLinkService>();

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
                app.UseHsts();
            }

            app.UseHttpsRedirection();
            app.UseDefaultFiles();
            app.UseStaticFiles();
            app.UseSwagger(c =>
                {
                    c.RouteTemplate = "openapi/{documentName}/openapi.json";
                })
                .UseSwaggerUI(c =>
                {
                    // set route prefix to openapi, e.g. http://localhost:8080/openapi/index.html
                    c.RoutePrefix = "openapi";
                    //TODO: Either use the SwaggerGen generated OpenAPI contract (generated from C# classes)
                    //c.SwaggerEndpoint("/openapi/" + apiVersion  + "/openapi.json", "World Storage API");

                    //TODO: Or alternatively use the original OpenAPI contract that's included in the static files
                    c.SwaggerEndpoint("/openapi-original.json", "World Storage API Original");
                });
            app.UseRouting();
            app.UseEndpoints(endpoints =>
                {
                    endpoints.MapControllers();
                });
        }
    }
}
